from __future__ import print_function

'''
	import pathlib
	from os.path import dirname, join, normpath
	import sys
	
	this_directory = pathlib.Path (__file__).parent.resolve ()
	PID_file_path = normpath (join (this_directory, "gunicorn.process_ID"))

	import vessels.flask.gunicorn as flask_gunicorn
	flask_gunicorn.start (
		flask_module_path = "",
		port = 54031,
		workers = 4
	)
'''

'''
	flask_gunicorn.start (
		PID_file_path = PID_file_path,
		flask_def = start,
		port = 54031,
		workers = 4
	)
'''


import builtins

import os
from os.path import dirname, join, normpath

import vessels.flask.gunicorn.settings as flask_gunicorn_settings
import vessels.flask.gunicorn.import_custom as import_custom
	
def start (
	PID_file_path = "",
	flask_module_path = "",

	workers = 4,
	port = 54031
):
	if (len (PID_file_path) == 0):
		settings = flask_gunicorn_settings.retrieve ()
		PID_file_name = settings ["PID file name"]
	
		PID_file_path = normpath (join (os.getcwd (), PID_file_name));

	#assert (type (flask_module_path) == str)
	#assert (len (flask_module_path) >= 1)
	
	assert (len (PID_file_path) >= 1)

	print (
		PID_file_path,
		flask_module_path,

		workers,
		port
	)
	
	flask_module = import_custom.start (flask_module_path);
	
	from gunicorn.app.base import Application
	class FlaskApplication (Application):
		def init (self, parser, opts, args):
			#
			#	This is run in another process presumably
			#
		
			return {			
				'bind': f'0.0.0.0:{ port }',
				'workers': int (workers),
				
				'daemon': True,
				'pidfile': PID_file_path
			}

		def load (self):
			#
			#	This is run in another process presumably
			#
		
			#return flask_module_path ()
			return flask_module.start ()

	print ('gunicorn is starting')

	application = FlaskApplication ()
	application.run ()







'''
	import vessels.SSH.connect as SSH_connect

	from os.path import dirname, join, normpath
	import pathlib
	import sys

	this_directory = pathlib.Path (__file__).parent.resolve ()
	private_key_path = normpath (join (this_directory, "private key"))

	SSH_connect.splendidly ({
		"private key": "",
		"to": {
			"address": "164.92.112.195"
		}
	})
'''

import os

def splendidly (parameters):
	private_key = parameters ["private key"]
	to_address = parameters ["to"] ["address"]

	script = " ".join ([
		"ssh -i",
		f"'{ private_key }'",
		f"root@{ to_address }"
	])

	print (script);
	
	os.system (script)






'''
	import vessels.flask.gunicorn.settings as flask_gunicorn_settings
	settings = flask_gunicorn_settings.retrieve ()
	PID_file_name = settings ["PID file name"]

'''

def retrieve ():
	return {
		"PID file name": "gunicorn.process_ID"
	}



'''
import vessels.inventory.itemize as itemize_inventory
itemize_inventory.start ()
'''

import glob
import json

def start (directories):
	inventory = {}

	for directory in directories:
		inventory_glob_string = folder + "/**/*"
		inventory_paths = glob.glob (folder + "/**/*", recursive = True)
		
		for path in inventory_paths:
			inventory [ path.split (folder + "/") [1] ] = path;
		
	print ("inventory:", json.dumps (inventory, indent = 4))
		
	return inventory



'''
import vessels.flask.gunicorn.import_custom as import_custom
flask_module = import_custom.start ("/structures/thruster.py")
vehicle.start ()
'''

from importlib.machinery import SourceFileLoader
import inspect
import os

	
def start (module_path):	
	print ("custom import:", module_path)

	if (module_path [ 0 ] == "/"):
		full_path = module_path;
		
	else:
		file_of_caller_function = os.path.abspath (
			(inspect.stack () [2]) [1]
		)
		directory_of_caller_function = os.path.dirname (
			file_of_caller_function
		)	
		
		full_path = os.path.normpath (directory_of_caller_function + "/" + module_path)

	return SourceFileLoader (full_path, full_path).load_module ()






'''
import vessels.flask.gunicorn.process_ID_path as read_process_ID_path
read_process_ID_path.enthusiastically ()
'''

'''
def retrieve ():
	import pathlib
	from os.path import dirname, join, normpath
	import sys
	
	this_directory = pathlib.Path (__file__).parent.resolve ()
	return normpath (join (this_directory, "gunicorn.process_ID"))
'''	
	
import pathlib
from os.path import dirname, join, normpath
import sys


import os

	
import os
def enthusiastically (
	trail = ""
):
	if (os.path.isfile (trail)):
		FP = open (trail, "r")
		return FP.read ().strip ()
		
	else:
		raise Exception ("The gunicorn process ID file was not found.")

	



'''
	import vessels.proxies.HA.install.Fedora as install_HA_proxy
	install_HA_proxy.charismatically ()
'''

def charismatically ():
	lyrics = "dnf install haproxy -y"
	
	print ("about to run:", lyrics)
	
	import os
	os.system (lyrics)





'''

'''


def start ():
	from flask import Flask

	app = Flask (__name__)

	@app.route ("/")
	def hello ():
		return "$"
		
	@app.route ('/', defaults={'path': ''})
	@app.route ('/<path:path>')
	def catch_all (path):
		return 'You want path: %s' % path
		
	return app;
	







'''
	priorities:		
		https://pypi.org/project/simple-term-menu/
'''

import os
from os.path import dirname, join, normpath

def clique ():
	import click
	@click.group ()
	def group ():
		pass
	
	
	import click
	@click.command ("shares")
	def shares ():
		import pathlib
		from os.path import dirname, join, normpath
		this_folder = pathlib.Path (__file__).parent.resolve ()

		import shares
		shares.start ({
			"directory": str (this_folder),
			"extension": ".s.HTML",
			"relative path": str (this_folder)
		})
	group.add_command (shares)
	
	'''
		fleet flask-gunicorn-start --port 
	'''
	import click
	@click.command ("flask-gunicorn-start")
	@click.option ('--port')
	@click.option ('--flask-module-path')
	def flask_gunicorn_start (port, flask_module_path):	
		flask_module_path = normpath (join (os.getcwd (), flask_module_path));
	
		import vessels.flask.gunicorn as flask_gunicorn
		flask_gunicorn.start (
			flask_module_path = flask_module_path
		)
		
	group.add_command (flask_gunicorn_start)

	import click
	@click.command ("flask-gunicorn-stop")
	def flask_gunicorn_stop ():	
		import vessels.flask.gunicorn.stop as stop_flask_gunicorn
		stop_flask_gunicorn.solidly ()
	group.add_command (flask_gunicorn_stop)


	import click
	@click.command ("HA-install-on-fedora")
	def HA_install_on_fedora ():	
		import vessels.proxies.HA.install.Fedora as install_HA_proxy
		install_HA_proxy.charismatically ()
	group.add_command (HA_install_on_fedora)

	import click
	@click.command ("HA-build-papers")
	def build_papers ():	
		import vessels.proxies.HA.SSL as HA_SSL
		HA_SSL.build_papers (
			certificate_path = "/etc/haproxy/SSL/certificate.pem",
			key_path = "/etc/haproxy/SSL/certificate.pem.key"
		)
	group.add_command (build_papers)

	import click
	@click.command ("HA-HTTPS-HTTP")
	@click.option ('--to-port')
	@click.option ('--workers', default = 4)
	def HA_HTTPS_HTTP (to_port, workers):	
		workers = int (workers)
		
		to_addresses = []
		s = 1
		while (s <= workers):
			to_addresses.append (f"0.0.0.0:{ to_port }")
			s += 1

		import vessels.proxies.HA.configs.HTTPS_to_HTTP as HA_HTTPS_to_HTTP
		HA_HTTPS_to_HTTP.build (
			start = "yes",

			SSL_certificate_path = "/etc/haproxy/SSL/certificate.pem",
			config_path = "/etc/haproxy/haproxy.cfg",
			
			to_addresses = to_addresses
		)
	group.add_command (HA_HTTPS_HTTP)

	group ()




'''
	import vessels.flask.gunicorn.stop as stop_flask_gunicorn
	stop_flask_gunicorn.solidly (
		PID_file_path = ""
	)
'''


import vessels.flask.gunicorn.process_ID_path as read_process_ID_path
import vessels.flask.gunicorn.settings as flask_gunicorn_settings
	

import os
from os.path import dirname, join, normpath
import pathlib
import psutil
import sys
	
def solidly (
	PID_file_path = ""
):
	if (len (PID_file_path) == 0):
		settings = flask_gunicorn_settings.retrieve ()
		PID_file_name = settings ["PID file name"]
	
		PID_file_path = normpath (join (os.getcwd (), PID_file_name));

	assert (type (PID_file_path) == str)
	assert (len (PID_file_path) >= 1)

	process_ID = int (
		read_process_ID_path.enthusiastically (
			trail = PID_file_path
		)
	)

	p = psutil.Process (process_ID)
	p.terminate ()  #or p.kill()
	
	print ("p:", p)